package connector;

public class ConnectionParameters {
    
    public ConnectionParameters(
            String pHost,
            Integer pPort,
            String pDatabase,
            String pLogin,
            String pPassword )
            throws IllegalArgumentException {
        
        if( pHost == null ) {
            throw new IllegalArgumentException( "Illegal Host argument: is null." );
        }
        if( pHost.trim().length() == 0 ) {
            throw new IllegalArgumentException( "Illegal Host argument: is empty." );
        }
        if( pPort == null ) {
            throw new IllegalArgumentException( "Illegal Port argument: is null." );
        }
        if( pPort <= 0 ) {
            throw new IllegalArgumentException( "Illegal Port argument: is less then zero." );
        }
        if( pDatabase == null ) {
            throw new IllegalArgumentException( "Illegal Database argument: is null." );
        }
        if( pDatabase.trim().length() == 0 ) {
            throw new IllegalArgumentException( "Illegal Database argument: is empty." );
        }
        if( pLogin == null ) {
            throw new IllegalArgumentException( "Illegal Login argument: is null." );
        }
        if( pLogin.trim().length() == 0 ) {
            throw new IllegalArgumentException( "Illegal Login argument: is empty." );
        }
        if( pPassword == null ) {
            throw new IllegalArgumentException( "Illegal Password argument: is null." );
        }
        if( pPassword.trim().length() == 0 ) {
            throw new IllegalArgumentException( "Illegal Password argument: is empty." );
        }
        
        mHost = new String( pHost.trim() );
        mPort = new Integer( pPort );
        mDatabase = new String( pDatabase.trim() );
        mLogin = new String( pLogin.trim() );
        mPassword = new String( pPassword );
    }
    
    public ConnectionParameters(
            ConnectionParameters pPrototype )
            throws IllegalArgumentException {
        
        if( pPrototype == null ) {
            throw new IllegalArgumentException( "Illegal Prototype argument: is null." );
        }
        
        mHost = new String( pPrototype.mHost.trim() );
        mPort = new Integer( pPrototype.mPort );
        mDatabase = new String( pPrototype.mDatabase.trim() );
        mLogin = new String( pPrototype.mLogin.trim() );
        mPassword = new String( pPrototype.mPassword );
    }
    
    public boolean setHost(
            String pHost ) {
        
        if( pHost == null ) {
            return false;
        }
        if( pHost.trim().length() == 0 ) {
            return false;
        }
        mHost = new String( pHost.trim() );
        return true;
    }
    
    public String getHost() {
        
        return new String( mHost );
    }
    
    public boolean setPort(
            Integer pPort ) {
        
        if( pPort == null ) {
            return false;
        }
        if( pPort <= 0 ) {
            return false;
        }
        mPort = new Integer( pPort );
        return true;
    }
    
    public Integer getPort() {
        
        return new Integer( mPort );
    }
    
    public boolean setDatabase(
            String pDatabase ) {
        
        if( pDatabase == null ) {
            return false;
        }
        if( pDatabase.trim().length() == 0 ) {
            return false;
        }
        mDatabase = new String( pDatabase.trim() );
        return true;
    }
    
    public String getDatabase() {
        
        return new String( mDatabase );
    }
    
    public boolean setLogin(
            String pLogin ) {
        
        if( pLogin == null ) {
            return false;
        }
        if( pLogin.trim().length() == 0 ) {
            return false;
        }
        mLogin = new String( pLogin.trim() );
        return true;
    }
    
    public String getLogin() {
        
        return new String( mLogin );
    }
    
    public boolean setPassword(
            String pPassword ) {
        
        if( pPassword == null ) {
            return false;
        }
        if( pPassword.trim().length() == 0 ) {
            return false;
        }
        mPassword = new String( pPassword );
        return true;
    }
    
    public String getPassword() {
        
        return new String( mPassword );
    }
    
    public boolean equals(
            Object pInstance ) {
        
        if( pInstance == null ) {
            return false;
        }
        if( pInstance instanceof ConnectionParameters ) {
            ConnectionParameters lInstance = ( ConnectionParameters )pInstance;
            if( ( mHost.equals( lInstance.mHost ) ) && 
                    ( mPort.equals( lInstance.mPort ) ) &&
                    ( mDatabase.equals( lInstance.mDatabase ) ) &&
                    ( mLogin.equals( lInstance.mLogin ) ) &&
                    ( mPassword.equals( lInstance.mPassword ) ) ) {
                return true;
            } else {
                return false;
            }
        } else {
            return false;
        }
    }
    
    public ConnectionParameters clone() {
        
        ConnectionParameters lClone = null;
        try {
            lClone = new ConnectionParameters( this );
        } finally {
            
        }
        return lClone;
    }
    
    private String mHost;
    private Integer mPort;
    private String mDatabase;
    private String mLogin;
    private String mPassword;
}