package connector;

import java.io.File;

import org.w3c.dom.*;
import javax.xml.parsers.*;

public final class ConnectionParametersXMLReader {
    
    private ConnectionParametersXMLReader() {
        
        mParametersFilePath = null;
    }
    
    public static ConnectionParametersXMLReader getInstance() {
        
        return mInstance;
    }
    
    public void setParametersFilePath(
            String pParametersFilePath ) {
        
        if( pParametersFilePath == null ) {
            mParametersFilePath = null;
            return;
        }
        if( pParametersFilePath.trim().length() == 0 ) {
            mParametersFilePath = null;
            return;
        }
        mParametersFilePath = new String( pParametersFilePath );
    }
    
    public String getParametersFilePath() {
        
        if( mParametersFilePath != null ) {
            return new String( mParametersFilePath );
        } else {
            return null;
        }
    }
    
    public ConnectionParameters readParameters() {
        
        if( mParametersFilePath == null ) {
            return null;
        }
        File lParametersFile = new File( mParametersFilePath );
        if( ! lParametersFile.exists() ) {
            return null;
        }
        
        Document lXMLDocument = null;
        DocumentBuilderFactory lFactory = DocumentBuilderFactory.newInstance();
        try {
            DocumentBuilder lBuilder = lFactory.newDocumentBuilder();
            lXMLDocument = lBuilder.parse( lParametersFile );
        } catch (Exception eFailed ) {
            return null;
        }
        
        ConnectionParameters lParameters = null;
        String lHost = null;
        Integer lPort = null;
        String lDatabase = null;
        String lLogin = null;
        String lPassword = new String( "fake password" );
        
        Element lRoot = lXMLDocument.getDocumentElement();
        NodeList lConnectionNodes = lRoot.getChildNodes();
        
        for( int iNode = 0; iNode < lConnectionNodes.getLength(); iNode++ ) {
            Node lNode = lConnectionNodes.item( iNode );
            if( lNode instanceof Element ) {
                Element lElement = ( Element )lNode;
                if( lElement.getTagName().equalsIgnoreCase( "host" ) ) {
                    Text lTextElement = ( Text )lElement.getFirstChild();
                    lHost = new String( lTextElement.getData().trim() );
                } else if( lElement.getTagName().equalsIgnoreCase( "port" ) ) {
                    Text lTextElement = ( Text )lElement.getFirstChild();
                    lPort = new Integer( lTextElement.getData().trim() );
                } else if( lElement.getTagName().equalsIgnoreCase( "database" ) ) {
                    Text lTextElement = ( Text )lElement.getFirstChild();
                    lDatabase = new String( lTextElement.getData().trim() );
                } else if( lElement.getTagName().equalsIgnoreCase( "login" ) ) {
                    Text lTextElement = ( Text )lElement.getFirstChild();
                    lLogin = new String( lTextElement.getData().trim() );
                }
            }
        }
        
        try {
            lParameters = new ConnectionParameters(
                    lHost,
                    lPort,
                    lDatabase,
                    lLogin,
                    lPassword );
        } finally {
        }
        return lParameters;
    }
    
    private static ConnectionParametersXMLReader mInstance = new ConnectionParametersXMLReader();
    
    private String mParametersFilePath;
}