package publications;

import java.util.Vector;

import java.io.File;
import java.io.IOException;

public abstract class Publication
    implements Viewable {
    
    public Publication(
            String pTitle,
            Vector< Author > pAuthors,
            Vector< Bookmark > pBookmarks )
            throws IllegalArgumentException {
        
        if( pTitle == null ) {
            throw new IllegalArgumentException( "Illegal Title argument: is null." );
        }
        if( pTitle.trim().length() == 0 ) {
            throw new IllegalArgumentException( "Illegal Title argument: is empty." );
        }
        if( pAuthors == null ) {
            throw new IllegalArgumentException( "Illegal Author argument: is null." );
        }
        if( pBookmarks == null ) {
            throw new IllegalArgumentException( "Illegal Bookmarks argument: is null." );
        }
        mTitle = new String( pTitle );
        mAuthors = ( Vector< Author > )pAuthors.clone();
        mBookmarks = ( Vector< Bookmark > )pBookmarks.clone();
    }
    
    public Publication(
            Publication pPrototype )
            throws IllegalArgumentException {
        
        if( pPrototype == null ) {
            throw new IllegalArgumentException( "Illegal Prototype argument: is null." );
        }
        mTitle = new String( pPrototype.mTitle );
        mAuthors = ( Vector< Author > )pPrototype.getAuthors().clone();
        mBookmarks = new Vector< Bookmark >();
        for( int iBookmark = 0; iBookmark < pPrototype.bookmarks(); iBookmark++ ) {
            mBookmarks.add( pPrototype.getBookmark( iBookmark ) );
        }
    }
    
    public boolean isViewable() {
        
        return false;
    }
    
    public boolean viewExternally() {
        
        if( mExternalViewer == null ) {
            return false;
        }
        if( mDocument == null ) {
            return false;
        }
        File lExternalViewerFile = new File( mExternalViewer );
        if( ! lExternalViewerFile.exists() ) {
            return false;
        }
        File lDocumentFile = new File( mDocument );
        if( ! lDocumentFile.exists() ) {
            return false;
        }
        
        String[] lCommand = new String[2];
        lCommand[0] = new String( lExternalViewerFile.getAbsolutePath() );
        lCommand[1] = new String( lDocumentFile.getAbsolutePath() );
        
        try {
            Runtime.getRuntime().exec( lCommand );
            return true;
        } catch ( IOException eFailed ) {
            return false;
        }
    }
    
    public void setExternalViewerFilePath(
            String pViewer ) {
        
        if( pViewer != null ) {
            mExternalViewer = new String( pViewer );
        } else {
            mExternalViewer = null;
        }
    }
    
    public String getExternalViewerFilePath() {
        
        if( mExternalViewer != null ) {
            return new String( mExternalViewer );
        } else {
            return null;
        }
    }
    
    public void setDocumentFilePath(
            String pDocument ) {
        
        if( pDocument != null ) {
            mDocument = new String( pDocument );
        } else {
            mDocument = null;
        }
    }
    
    public String getDocumentFilePath() {
        
        if( mDocument != null ) {
            return new String( mDocument );
        } else {
            return null;
        }
    }
    
    public boolean setTitle(
            String pTitle ) {
        
        if( pTitle == null ) {
            return false;
        }
        if( pTitle.trim().length() == 0 ) {
            return false;
        }
        mTitle = new String( pTitle );
        return true;
    }
    
    public String getTitle() {
        
        return new String( mTitle );
    }
    
    public int bookmarks() {
        
        return mBookmarks.size();
    }
    
    public Bookmark getBookmark( 
            int pBookmarkNumber ) {
        
        if( ( pBookmarkNumber >= 0) && ( pBookmarkNumber < mBookmarks.size() ) ) {
            return mBookmarks.get( pBookmarkNumber ).clone();
        } else {
            return null;
        }
    }
    
    public abstract PublicationType getType();
    
    public boolean setAuthors(
            Vector< Author > pAuthor ) {
        
        if( pAuthor == null ) {
            return false;
        }
        mAuthors = ( Vector< Author > )pAuthor.clone();
        return true;
    }
    
    public Vector< Author > getAuthors() {
        
        return ( Vector< Author > )mAuthors.clone();
    }
    
    private String mTitle;
    private Vector< Bookmark > mBookmarks;
    private Vector< Author > mAuthors;
    
    private String mExternalViewer;
    private String mDocument;
}
