package readers;

import java.sql.Connection;
import java.sql.SQLException;
import java.sql.Statement;
import java.sql.ResultSet;

import publications.*;

import java.util.Vector;
import java.util.StringTokenizer;

public class DatabaseReader
    implements Reader {
    
    public DatabaseReader(
            Connection pConnection ) {
        
        mConnection = pConnection;
    }
    
    public Publication getPublication( 
            Bookmark pBookmark ) {
        
        if( ! ready() ) {
            return null;
        }
        
        if( ! ( pBookmark instanceof DatabaseBookmark ) ) {
            return null;
        }
        DatabaseBookmark lBookmark = ( DatabaseBookmark )pBookmark;
        
        Integer lKey = lBookmark.getKey();
        
        Publication lPublication = null;
        
        if( lBookmark.getType() == BookmarkType.PublicationBookmark ) {
            String lPublicationQuery = new String(
                    "SELECT p.title, p.type, p.id, p.path, ev.path " + 
                    "FROM publication p, external_viewer ev " + 
                    "WHERE ev.id = p.viewer " + 
                    " AND p.id = " +
                    lKey.toString() + ";" );
            ResultSet lPublicationResults = executeQuery( lPublicationQuery );
            String lTitle = null;
            Integer lType = null;
            String lPublicationPath = null;
            String lViewerPath = null;
            Integer lPublicationKey = null;
            try {
                if( lPublicationResults.next() ) {
                    lTitle = new String( lPublicationResults.getString( "p.title" ) );
                    lType = new Integer( lPublicationResults.getString( "p.type" ) );
                    lPublicationKey = new Integer( lPublicationResults.getString( "p.id" ) );
                    lPublicationPath = new String( lPublicationResults.getString( "p.path" ) );
                    lViewerPath = new String( lPublicationResults.getString( "ev.path" ) );
                }  else {
                    return null;
                }
            } catch ( SQLException eFailed ) {
                return null;
            }

            String lPublicationsBookmarksQuery = new String( 
                    "SELECT p.id, p.title " +
                    "FROM publication p, publication_bookmarks tb " +
                    "WHERE tb.publication = " + lPublicationKey.toString()  +  
                    " AND p.id = tb.bookmark " +
                    "ORDER BY p.title;" );
            ResultSet lPublicationsBookmarksResult = executeQuery( lPublicationsBookmarksQuery );
            if( lPublicationsBookmarksResult == null ) {
                return null;
            }
            Vector< Bookmark > lBookmarks = new Vector< Bookmark >();
            try {
                while( lPublicationsBookmarksResult.next() ) {
                    lBookmarks.add(
                            new DatabaseBookmark( 
                                lPublicationsBookmarksResult.getString( "p.title" ),
                                BookmarkType.PublicationBookmark,
                                lPublicationsBookmarksResult.getInt( "p.id" ) ) );
                }
            } catch( SQLException eFailed ) {
            }
            String lAuthorsBookmarksQuery = new String( 
                    "SELECT a.id, a.name, a.surname " + 
                    "FROM author a, publication_author_bookmarks pab " + 
                    "WHERE pab.author = a.id AND " + 
                    "pab.publication = " + lPublicationKey.toString()  + 
                    " ORDER BY a.surname;" );
            ResultSet lAuthorsBookmarksResult = executeQuery( lAuthorsBookmarksQuery );
            if( lAuthorsBookmarksResult == null ) {
                return null;
            }
            try {
                while( lAuthorsBookmarksResult.next() ) {
                    lBookmarks.add(
                            new DatabaseBookmark( 
                                "  " + lAuthorsBookmarksResult.getString( "a.surname" ) + " " + lAuthorsBookmarksResult.getString( "a.name" ) ,
                                BookmarkType.AuthorBookmark,
                                lAuthorsBookmarksResult.getInt( "a.id" ) ) );
                }
            } catch( SQLException eFailed ) {
            }
            String lAuthorQuery = new String(
                    "SELECT a.name, a.surname " +
                    "FROM author a, publication_authors pa " +
                    "WHERE pa.publication = " + lPublicationKey.toString() +
                    " AND pa.author = a.id " +
                    "ORDER BY a.surname;" );
            ResultSet lPublicationAuthor = executeQuery( lAuthorQuery );
            Vector< Author > lAuthors = new Vector< Author >();
            try {
                while( lPublicationAuthor.next() ) {
                    lAuthors.add( new Author(
                            lPublicationAuthor.getString( "a.name" ),
                            lPublicationAuthor.getString( "a.surname" ) ) );
                }
            } catch( SQLException eFailed ) {
                System.out.println( "Topics table corrupted." );
            }
            if( lType == 1 ) {
                lPublication = new Book(
                        lTitle,
                        lAuthors,
                        lBookmarks );
                lPublication.setDocumentFilePath( lPublicationPath );
                lPublication.setExternalViewerFilePath( lViewerPath );
            } else if( lType == 2 ) {
                lPublication = new Bind(
                        lTitle,
                        lAuthors,
                        lBookmarks );
            }
        } else if( lBookmark.getType() == BookmarkType.AuthorBookmark ) {
            String lPublicationsBookmarksQuery = new String( 
                    "SELECT p.id, p.title " +
                    "FROM publication p, publication_authors pa " +
                    "WHERE pa.author = " + lKey.toString()  +  
                    " AND p.id = pa.publication " +
                    "ORDER BY p.title;" );
            ResultSet lPublicationsBookmarksResult = executeQuery( lPublicationsBookmarksQuery );
            if( lPublicationsBookmarksResult == null ) {
                return null;
            }
            Vector< Bookmark > lBookmarks = new Vector< Bookmark >();
            try {
                while( lPublicationsBookmarksResult.next() ) {
                    lBookmarks.add(
                            new DatabaseBookmark( 
                                lPublicationsBookmarksResult.getString( "p.title" ),
                                BookmarkType.PublicationBookmark,
                                lPublicationsBookmarksResult.getInt( "p.id" ) ) );
                }
            } catch( SQLException eFailed ) {
            }
            String lAuthorQuery = new String(
                    "SELECT a.name, a.surname " +
                    "FROM author a " +
                    "WHERE a.id = " + lKey.toString() );
            ResultSet lPublicationAuthor = executeQuery( lAuthorQuery );
            Vector< Author > lAuthors = new Vector< Author >();
            try {
                while( lPublicationAuthor.next() ) {
                    lAuthors.add( new Author(
                            lPublicationAuthor.getString( "a.name" ),
                            lPublicationAuthor.getString( "a.surname" ) ) );
                }
            } catch( SQLException eFailed ) {
                System.out.println( "Topics table corrupted." );
            }
            lPublication = new Bind (
                    " ",
                    lAuthors,
                    lBookmarks );
        }
        return lPublication;
    }
    
    public Vector< String > getTopics() {

        if( ! ready() ) {
            return null;
        }
        
        Vector< String > lTopics = new Vector< String >();
        String lQuery = new String( 
                "SELECT title " +
                "FROM publication " +
                "WHERE type = 2 " +
                "ORDER BY title;" );
        ResultSet lDatabaseTopics = executeQuery( lQuery );
        try {
            while( lDatabaseTopics.next() ) {
                lTopics.add( lDatabaseTopics.getString( "title" ) );
            }
        } catch( SQLException eFailed ) {
        }
        return lTopics;
    }
    
    public Publication getTopic(
            String pTopic ) {
        
        if( ! ready() ) {
            return null;
        }
        
        String lBindIdQuery = new String(
                "SELECT id " +
                "FROM publication " +
                "WHERE title LIKE '%" + pTopic +
                "%' AND type = 2;" );
        ResultSet lBindIdResult = executeQuery( lBindIdQuery );
        if( lBindIdResult == null ) {
            return null;
        }
        Integer lBindId = null;
        try {
            if( lBindIdResult.next() ) {
                lBindId = new Integer( lBindIdResult.getInt( "id" ) );
            } else {
                return null;
            }
        } catch( SQLException eFailed ) {
            return null;
        }
        
        Bookmark lBookmark = new DatabaseBookmark(
                pTopic,
                BookmarkType.PublicationBookmark,
                lBindId );
        
        return getPublication( lBookmark );
    }
    
    public Publication getAuthorList() {
        
        Vector< Bookmark > lBookmarks = new Vector< Bookmark >();
        String lAuthorsBookmarksQuery = new String( 
                "SELECT id, name, surname " + 
                "FROM author " + 
                " ORDER BY surname, name;" );
        ResultSet lAuthorsBookmarksResult = executeQuery( lAuthorsBookmarksQuery );
        if( lAuthorsBookmarksResult == null ) {
            return null;
        }
        try {
            while( lAuthorsBookmarksResult.next() ) {
                lBookmarks.add(
                        new DatabaseBookmark( 
                            lAuthorsBookmarksResult.getString( "surname" ) + " " + lAuthorsBookmarksResult.getString( "name" ) ,
                            BookmarkType.AuthorBookmark,
                            lAuthorsBookmarksResult.getInt( "id" ) ) );
            }
        } catch( SQLException eFailed ) {
        }
        Vector< Author > lAuthors = new Vector< Author >();
        Publication lPublication = new Bind(
                " ",
                lAuthors,
                lBookmarks );
        return lPublication;
    }
    
    public Publication getBookList() {
        
        Vector< Bookmark > lBookmarks = new Vector< Bookmark >();
        String lBooksBookmarksQuery = new String( 
                "SELECT id, title " + 
                "FROM publication " + 
                "WHERE type = 1 " +
                " ORDER BY title;" );
        ResultSet lBooksBookmarksResult = executeQuery( lBooksBookmarksQuery );
        if( lBooksBookmarksResult == null ) {
            return null;
        }
        try {
            while( lBooksBookmarksResult.next() ) {
                lBookmarks.add(
                        new DatabaseBookmark( 
                            lBooksBookmarksResult.getString( "title" ),
                            BookmarkType.PublicationBookmark,
                            lBooksBookmarksResult.getInt( "id" ) ) );
            }
        } catch( SQLException eFailed ) {
        }
        Vector< Author > lAuthors = new Vector< Author >();
        Publication lPublication = new Bind(
                " ",
                lAuthors,
                lBookmarks );
        return lPublication;        
    }
    
    public Publication search( 
            String pConditions,
            boolean pByAuthor,
            boolean pByPublication ) {
        
        Vector< Author > lBindAuthors = new Vector< Author >();
        Vector< Bookmark > lBookmarks = new Vector< Bookmark >();
        String lTitle = new String( "    '" + pConditions + "' " );
        StringTokenizer lConditions = new StringTokenizer( pConditions, " " );
        String lPublicationQuery = new String(
                "SELECT title, id " + 
                "FROM publication " + 
                " WHERE " );
        String lAuthorQuery = new String(
                "SELECT name, surname, id " +
                "FROM author " +
                "WHERE " );
        if( lConditions.hasMoreTokens() ) {
            String lToken = lConditions.nextToken();
            lPublicationQuery = lPublicationQuery + " title LIKE '%"  + lToken + "%' ";
            lAuthorQuery = lAuthorQuery + " name LIKE '%" + lToken + "%' OR surname LIKE '%" + lToken + "%' ";
        }
        while( lConditions.hasMoreTokens() ) {
            String lToken = lConditions.nextToken();
            lPublicationQuery = lPublicationQuery + " OR title LIKE '%"  + lToken + "%' ";
            lAuthorQuery = lAuthorQuery + " OR name LIKE '%" + lToken + "%' OR surname LIKE '%" + lToken + "%' ";
        }
        lPublicationQuery  = lPublicationQuery + " ORDER BY title;";
        lAuthorQuery = lAuthorQuery + " ORDER BY surname, name;";
        if( pByAuthor ) {
            ResultSet lAuthors = executeQuery( lAuthorQuery );
            if( lAuthors != null ) {
                try {
                    while( lAuthors.next() ) {
                        lBookmarks.add(
                                new DatabaseBookmark( 
                                    lAuthors.getString( "surname" ) + " " + lAuthors.getString( "name" ) ,
                                    BookmarkType.AuthorBookmark,
                                    lAuthors.getInt( "id" ) ) );
                    }
                } catch( SQLException eFailed ) {
                }
            }
        }
        if( pByPublication ) {
            ResultSet lPublications = executeQuery( lPublicationQuery );
            if( lPublications != null ) {
                try {
                    while( lPublications.next() ) {
                        lBookmarks.add(
                                new DatabaseBookmark( 
                                    lPublications.getString( "title" ),
                                    BookmarkType.PublicationBookmark,
                                    lPublications.getInt( "id" ) ) );
                    }
                } catch( SQLException eFailed ) {
                }
            }
        }
        Publication lSearchResults = new Bind(
                lTitle,
                lBindAuthors,
                lBookmarks );
        return lSearchResults;        
    }
    
    public boolean ready() {
        
        if( mConnection == null ) {
            return false;
        } else {
            return true;
        }
    }

    private ResultSet executeQuery(
            String pQuery ) {
        
        if( pQuery == null ) {
            return null;
        }
        Statement lStatement = null;
        ResultSet lResult = null;
        try {
            lStatement = mConnection.createStatement();
            lResult = lStatement.executeQuery( pQuery );
            return lResult;
        } catch( Exception eQueryException ) {
                if ( lResult != null) {
                    try {
                        lResult.close();
                    } catch ( Exception lClosingFailed ) {
                        lResult = null;
                    }
                }
                if ( lStatement != null) {
                    try {
                        lStatement.close();
                    } catch ( Exception lClosingFailed ) {
                        lStatement = null;
                    }
                }
        }        
        return null;
    }
    
    private Connection mConnection;
}