package travellers;

import java.util.Vector;

import readers.Reader;

import publications.Publication;
import publications.Bookmark;

public class Traveller {
    
    public Traveller(
            Reader pReader )
            throws IllegalArgumentException {
        
        if( pReader == null ) {
            throw new IllegalArgumentException( "Illegal Reader argument: is null." );
        }
        if( ! pReader.ready() ) {
            throw new IllegalArgumentException( "Illegal Reader argument: is corrupted." );
        }
        mReader = pReader;
        mStations = new Vector< TravellerStation >();
        mCurrentStation = -1;
    }
    
    public Vector< String > getTopics() {
        
        return mReader.getTopics();
    }
    
    public Publication travelToTopic(
            String pTopicName ) {
        
        Publication lPublication = mReader.getTopic( pTopicName );
        if( lPublication != null ) {
            addTravellerStation( new TopicTravellerStation( 
                    new String( pTopicName ) ) );
        }
        return lPublication;
    }
    
    public Publication getAuthorsList() {
        
        Publication lPublication = mReader.getAuthorList();
        if( lPublication != null ) {
            addTravellerStation( new AuthorListTravellerStation() );
        }
        return lPublication;
    }
    
    public Publication getBookList() {
        
        Publication lPublication = mReader.getBookList();
        if( lPublication != null ) {
            addTravellerStation( new BookListTravellerStation() );
        }
        return lPublication;
    }
    
    public Publication search(
            String pConditions,
            boolean pByAuthor,
            boolean pByPublication ) {
        
        Publication lPublication = mReader.search(
                pConditions,
                pByAuthor,
                pByPublication );
        if( lPublication != null ) {
            addTravellerStation( new SearchTravellerStation(
                    pConditions,
                    pByAuthor,
                    pByPublication ) );
        }
        return lPublication;
    }
    
    public Publication travelToBookmark(
            Bookmark pBookmark ) {
        
        Publication lPublication = mReader.getPublication( pBookmark );
        if( lPublication != null ) {
            addTravellerStation( new BookmarkTravellerStation( 
                    pBookmark.clone() ) );
        }
        return lPublication;
    }
    
    private void addTravellerStation( 
            TravellerStation pStation ) {
        
        if( mCurrentStation == -1 ) {
            mStations.add( pStation );
            mCurrentStation = 0;
        } else if( mCurrentStation == ( mStations.size() - 1 ) ) {
            mStations.add( pStation );
            mCurrentStation++;
        } else {
            while( mCurrentStation != ( mStations.size() - 1 ) ) {
                mStations.remove( mStations.size() - 1 );
            }
            mStations.add( pStation );
            mCurrentStation++;
        }
    }
    
    public Publication getCurrent() {
        
        return mStations.get( mCurrentStation ).getPublication( mReader );
    }
    
    public Publication travelToPrevious() {
        
        if( couldTravelToPrevious() ) {
            mCurrentStation--;
            return mStations.get( mCurrentStation ).getPublication( mReader );
        } else {
            return null;
        }
    }
    
    public boolean couldTravelToPrevious() {
        
        if( mCurrentStation > 0 ) {
            return true;
        } else {
            return false;
        }
    }
    
    public Publication travelToNext() {
        
        if( couldTravelToNext() ) {
            mCurrentStation++;
            return mStations.get( mCurrentStation ).getPublication( mReader );
        } else {
            return null;
        }
    }
    
    public boolean couldTravelToNext() {
        
        if( ( mCurrentStation < ( mStations.size() - 1 ) ) &&
                ( mCurrentStation >= 0 ) ) {
            return true;
        } else {
            return false;
        }
    }
    
    private Vector< TravellerStation > mStations;
    private int mCurrentStation;
    private Reader mReader;
}
